function [ powerMGArray,sinrMGArray,utilityMGArray,priceConvergeProcessArray,powerConvergeProcessArray,TotalIteNum]= ...
    funLogUADPGradientComp( M, B, pMax, n0, thetaArray, distanceArray, gridUnit,attenFactor,...
                    iniType,stopEps)


% Examine the total utility for M users with weighted logarithmic utility while 
% (1) All use maximum availble power
% (2) Users game distributedly by paying each other the deduction in interference,
%     with the best reponse function of the Modified Game, assuming the utility functions
%     are private infomation
%
% =========================================================================================
% Copyright notice:
% Related Journal Reference:
% 
% J. Huang, R. Berry and M. L. Honig, "Distributed Interference Compensation for Wireless Networks", 
% IEEE Journal on Selected Areas in Communications (JSAC), vol. 24, no. 5, pp. 1074-1084, May 2006
%
% Related Conference References:
% 
% (1) J. Huang, R. Berry and M. L. Honig, "Spectrum Sharing with Distributed Interference Compensation," 
%     IEEE Dynamic Spectrum Access Networks (DySPAN), Baltimore, MD, USA, November 2005
% (2) J. Huang, R. Berry and M. L. Honig, "Performance of Distributed Utility-based Power Control for Wireless Ad Hoc Networks," 
%     IEEE MILCOM, Atlantic City, NJ, USA, October 2005
% (3) J. Huang, R. Berry and M. L. Honig, "Distributed Interference Compensation for Multi-Channel Wireless Networks," 
%     Annual Allerton Conference on Communication, Control and Computing, Monticello, IL, USA, September, 2005
% (4) J. Huang, R. Berry and M. L. Honig, "A Game Theoretic Analysis of Distributed Power Control for Spread Spectrum Ad Hoc Networks," 
%     IEEE International Symposium on Information Theory (ISIT), Adelaide, Australia, September 2005
%
% All rights belong to Jianwei Huang (email: ADP.20.jianweiece@antichef.com), Randall Berry, Michael Honig.
%
% This simulation code can be freely modified and distributed, as long as the the copyright notice
% section is kept unchanged. Any comments/suggestions are welcome. However, the original authors of 
% the code are not responsible for any damages caused by this code.
%
% If this simulation code (or its variation) is used in adademic research, one or more of the 
% above journal/conference references should be appropriately cited. 
%
% Author: Jianwei Huang
% Current version: March 31, 2007
% ========================================================================================

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% M: number of pairs
% B: bandwidth
% pMax: maximum transmission power of each transmitter (assume same for
% every one)
% thetaArray: users' utility parameter vector
% distanceArray: the distance matrix between each transmitter and receiver (MxM)
% gridUnit: the length of a smallest grid
% attenFactor: the attenuation between two nodes is distance^(-attenFactor)
% hArray: path attenuation matrix (MxM)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%=================================================================================
% Author: Jianwei Huang
% Version 2: Jan. 7, 2005
% Version 3: April 10, 2005
%=================================================================================

hArray = (distanceArray*gridUnit).^(-attenFactor);
diagHArray = reshape(diag(hArray),1,M);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     ADP algorithm:
%     The next step power is chosen to maximize the surplus assuming fixed price from the other user
%     The next step price is chosen equaling to the current utility's derivate of interference
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% stopEps: the stop criotie when the price converge
% priceIteGapMax: the maximum differece between the current and previous prices
% priceCurrentArray: the price of current iteration
% powerCurrentArray: the power of current iteration
% iUpdate: number of iteration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% stopEps =  1e-4;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Price and power initilization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if iniType == 1; % Min price and Max power;   
	priceIniArray = stopEps*ones(1,M);
	powerIniArray = pMax*ones(1,M);
elseif iniType == 2; % Max price and Min power
	priceIniArray = thetaArray./(n0*B);
	powerIniArray = stopEps*ones(1,M);
else % Rand price and power    
	priceIniArray = thetaArray./(n0*B).*rand(1,M);
	powerIniArray = pMax*rand(1,M);
end;

priceCurrentArray = priceIniArray;
powerCurrentArray = powerIniArray;

% the maximum difference between powers in two rounds. Initial value set to 2*stopEps.
powerIteGapMax = 2*stopEps;

% the values of price and power during the process of convergence with dimension:
% (iUpdatemax X M)
priceConvergeProcessArray = zeros(1,M);
powerConvergeProcessArray = zeros(1,M);

iUpdate = 0;

while (powerIteGapMax > stopEps) | (iUpdate < 2); 
    
    iUpdate = iUpdate + 1;
    priceConvergeProcessArray(iUpdate,:) = priceCurrentArray;
    powerConvergeProcessArray(iUpdate,:) = powerCurrentArray;
    
%     priceArray(iUpdate,:) = thetaArray...
%         ./(n0*B+(powerCurrentArray*hArray-powerCurrentArray.*diagHArray));  
%     priceCurrentArray = priceArray(iUpdate,:);    
    
    powerArray(iUpdate,:) = min(pMax,...
        thetaArray./(priceCurrentArray*(hArray')-priceCurrentArray.*diagHArray));    
    powerIteGapMax = max(abs((powerArray(iUpdate,:)-powerCurrentArray))./powerCurrentArray);    
    powerCurrentArray = powerArray(iUpdate,:);  % use the latest power to update price
    
    
    priceArray(iUpdate,:) = thetaArray...
        ./(n0*B+(powerCurrentArray*hArray-powerCurrentArray.*diagHArray));
%     priceIteGapMax = max(abs((priceArray(iUpdate,:)-priceCurrentArray))./priceCurrentArray);
    
 %   powerCurrentArray = powerArray(iUpdate,:);   
    priceCurrentArray = priceArray(iUpdate,:);
    
    sinrArray(iUpdate,:) = powerCurrentArray.*diagHArray...
        ./(n0+(powerCurrentArray*hArray-powerCurrentArray.*diagHArray)./B);
    
    utilityArray (iUpdate,:)= thetaArray.*log(sinrArray(iUpdate,:));    
 
end;

powerMGArray = powerArray(iUpdate,:);
sinrMGArray = sinrArray(iUpdate,:);
utilityMGArray = utilityArray(iUpdate,:);
TotalIteNum = iUpdate;

